package net.ase.ingredientapp;

import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;
import org.json.JSONArray;
import org.json.JSONObject;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.database.Cursor;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;
import android.content.DialogInterface.OnCancelListener;

public class ActualizarBdActivity extends Activity {

	// Dialgo progress bar
	ProgressDialog pDialog;

	// Objetos view de la activity
	TextView txtUltAct;
	TextView txtTotalProdsNube;
	TextView txtTotalProdsDisp;
	TextView txtProdsPendRec;
	TextView txtPendCompartir;	
	ImageView btnActualizar;
	
	// Campos auxiliares del servicio web;
	int numProdsGlobalNube; //Nmero de productos en la bd global en la Nube
	int numProdsGlobal;  //Nmero de productos en la bd global en el dispositivo
	int numProdsLocal; //nmero de productos en la bd local pendientes de compartir
	String ultimaActualizacion; //fecha de la ultima actualizacion	
	int ultimoProdRecibido; //el id entero del ltimo producto recibido del servidor central
	private Producto[] productos=null; //Array para almacenar los productos
		
	// Para ejecutar los servicios REST asncronamente
	TareaAsincEstadoActulizacion stActualizacion;
	TareaAsincEnviarProdsLocal envProdsLocal;
	TareaAsincRecibirProdsGlobal rebProdsGlobal;
	TareaAsincInsertarProductosNuevos insNuevosProds;

	// creamos una instancia de la BD
	private AdaptadorBD db;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_actualizar_bd);

		// Referenciar los objetos de la vista
		txtUltAct = (TextView) findViewById(R.id.txtUltAct);
		btnActualizar = (ImageView) findViewById(R.id.btnActualizar);

		txtTotalProdsNube = (TextView) findViewById(R.id.txtTotalProdsNube);
		txtTotalProdsDisp = (TextView) findViewById(R.id.txtTotalProdsDisp);
		txtProdsPendRec = (TextView) findViewById(R.id.txtProdsPendRec);
		txtPendCompartir = (TextView) findViewById(R.id.txtPendCompartir);

		// Creamos una instancia de la BD
		db = new AdaptadorBD(this);
		db.abrir();
	
		// Mostrar resumen del estado
		stActualizacion = new TareaAsincEstadoActulizacion();
		stActualizacion.execute();
		
		
		btnActualizar.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				
				try {
					// La actualizacin se ejecuta en cuatro pasos uno tras otro en serie.
					// Cuando una tarea acaba ejecuta la siguiente al finalizarse ella misma.
					//1.- Compartir productos.
					//2.- Recibir nuevos productos
					//3.- Actualizar la base de datos del dispositivo
					//4.- Actualizar pantalla de resumen de nuevo.
					envProdsLocal = new TareaAsincEnviarProdsLocal();
					envProdsLocal.execute();
	
					// Informar de la fecha de la ltima ejecuci de la act.
					SimpleDateFormat df1 = new SimpleDateFormat("dd/MM/yyyy");
					String auxDate= df1.format(new Date());				


					db.setUltimaActualizacion(auxDate);
					
				}catch (Exception ex){
					Log.e("setUltimaActualizacion", "Error!", ex);
				} 
			}
		});

	}

	
	/**
	 * Tarea asncrona para enviar los productos creados por el usuario de forma
	 * local al servidor central.
	 */
	private class TareaAsincEnviarProdsLocal extends
			AsyncTask<Void, Integer, Boolean> {

		@Override
		protected Boolean doInBackground(Void... params) {
			
			try
			{
				numProdsGlobalNube=numProdsGlobalNube();
				// obtener todos los productos de la tabla local
				Cursor prodsLoc = db.obtenerTodosLosProductosLocal();
				
				if (prodsLoc != null) {
					if (prodsLoc.moveToFirst()){		
						do {

							if (compartirProducto(prodsLoc.getString(1), prodsLoc.getString(2),prodsLoc.getString(3)))
							{
								db.borrarProductoLocal(prodsLoc.getLong(0));
								publishProgress(1);
							}
		
							if (isCancelled())
								break;

						} while (prodsLoc.moveToNext());
						
					}

				}

				
			} catch (Exception ex) {
				Log.e("TareaAsincEnviarProdsLocal", "Error!", ex);
				
			} 
			return true;
		}

		@Override
		protected void onProgressUpdate(Integer... values) {
			int progreso = values[0].intValue();

			pDialog.setProgress(progreso);
			
		}

		@Override
		protected void onPreExecute() {

			pDialog = new ProgressDialog(ActualizarBdActivity.this);
			pDialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
			pDialog.setMessage(ActualizarBdActivity.this.getText(R.string.acvty_actualizarDb_Compartir));
			pDialog.setCancelable(true);
			pDialog.setMax(numProdsLocal);
			
			
			pDialog.setOnCancelListener(new OnCancelListener() {
				@Override
				public void onCancel(DialogInterface dialog) {
					TareaAsincEnviarProdsLocal.this.cancel(true);
				}
			});

			pDialog.setProgress(0);
			pDialog.show();
		}

		@Override
		protected void onPostExecute(Boolean result) {

			pDialog.dismiss();
			pDialog=null;			

			
			if (result) {

				// Actualizar contadores
				numProdsLocal=(int) db.numProductosLocal();
				txtTotalProdsNube.setText("" + numProdsGlobalNube);
				txtProdsPendRec.setText("" + (numProdsGlobalNube - numProdsGlobal));
				txtPendCompartir.setText("" + numProdsLocal );
				
				// Segundo paso de la actualizacin.
				rebProdsGlobal = new TareaAsincRecibirProdsGlobal();
				rebProdsGlobal.execute();				
				
				//Toast.makeText(ActualizarBdActivity.this, "Tarea finalizada!",Toast.LENGTH_SHORT).show();
			}


			
		}

		@Override
		protected void onCancelled() {
			//Toast.makeText(ActualizarBdActivity.this, "Tarea cancelada!",Toast.LENGTH_SHORT).show();
			pDialog.dismiss();
			pDialog=null;
		}

		/**
		 * Enva un producto local a la base da datos global para compartirlo
		 * con el resto de usuarios.
		 * 
		 * @param descripcion
		 *            la descripcin del producto.
		 * @param ingredientes
		 *            los ingredientes del producto.
		 * @return devuelve un booleano indicando si se pudo compartir el
		 *         producto.
		 */
		private boolean compartirProducto( String codigo,
				String descripcion, String ingredientes) {

			boolean resul = false;

			HttpClient httpClient = new DefaultHttpClient();

			HttpPost post = new HttpPost(
					"http://ingredientapp.azurewebsites.net/api/ProductosApi");
			post.setHeader("content-type", "application/json");

			try {

				// Construimos el objeto en formato JSON
				JSONObject dato = new JSONObject();

				// dato.put("id", Integer.parseInt(txtId.getText().toString()));
				dato.put("codigo", codigo);
				dato.put("descripcion", descripcion);
				dato.put("ingredientes", ingredientes);
				// dato.put("fabricante", Integer.parseInt(params[3]));

				StringEntity entity = new StringEntity(dato.toString(), HTTP.UTF_8);
				
				post.setEntity(entity);

				HttpResponse resp = httpClient.execute(post);

				// 201 Created .The request has been fulfilled and resulted in a
				// new resource being created2.
				if (resp.getStatusLine().getStatusCode() == 201) {
					resul = true;
									}
				else{
					resul=false;
				}

			} catch (Exception ex) {
				Log.e("ServicioRest", "Error!", ex);
				resul = false;
			}

			return resul;

		}

	}

	/**
	 * Tarea asncrona para recibir del servidor los productos que faltan en el
	 * dispositivo.
	 */
	private class TareaAsincRecibirProdsGlobal extends
			AsyncTask<Void, Integer, Boolean> {


		@Override
		protected Boolean doInBackground(Void... params) {

	    	boolean resul = true;
	    	
	    	HttpClient httpClient = new DefaultHttpClient();
			
			try
	        {
				
				

			    	HttpGet del = new HttpGet("http://ingredientapp.azurewebsites.net/api/ProductosApi/"+URLEncoder.encode("{from}","UTF-8")+"?id="+ultimoProdRecibido);	    	
					del.setHeader("content-type", "application/json");
					
		        	HttpResponse resp = httpClient.execute(del);
		        	String respStr = EntityUtils.toString(resp.getEntity());
		        	
		        	JSONArray respJSON = new JSONArray(respStr);
		        	
		        	productos = new Producto[respJSON.length()];
		        			
		        	for(int i=0; i<productos.length; i++)
		        	{
		        		JSONObject obj = respJSON.getJSONObject(i);
		        		productos[i]=new Producto();
			        	productos[i].setId(obj.getInt("id"));
			        	productos[i].setCodigo(obj.getString("codigo"));
			        	productos[i].setDescripcion(obj.getString("descripcion"));
			        	productos[i].setIngredientes(obj.getString("ingredientes"));
			        	productos[i].setFabricante(obj.getString("fabricante"));
		        	}
					
				
				
	        }
	        catch(Exception ex)
	        {
	        	Log.e("ServicioRest","Error!", ex);
	        	resul = false;
	        }
	 
	        return resul;
		}

		@Override
		protected void onProgressUpdate(Integer... values) {
			//int progreso = values[0].intValue();
			//pDialog.setProgress(progreso);
		}

		@Override
		protected void onPreExecute() {

			//calculamos el ltimo producto recibido del servidor - nube
			try
			{
				ultimoProdRecibido=(int)db.getUltimoProductoRecibido();
				
			} catch (Exception ex){
				Log.e("TareaAsincRecibirProdsGlobal onPreExecute", "Error!", ex);
			}

			pDialog=null;
			pDialog = new ProgressDialog(ActualizarBdActivity.this);
			pDialog.setMessage(ActualizarBdActivity.this.getText(R.string.acvty_actualizarDb_Recibir) + "" + (numProdsGlobalNube - numProdsGlobal));
			pDialog.setCancelable(true);
			pDialog.setIndeterminate(true);			
			
			pDialog.setOnCancelListener(new OnCancelListener() {
				@Override
				public void onCancel(DialogInterface dialog) {
					TareaAsincRecibirProdsGlobal.this.cancel(true);
				}
			});
			
			pDialog.show();
		}

		@Override
		protected void onPostExecute(Boolean result) {

			pDialog.dismiss();
			pDialog=null;


			
			
			if (result) {
				
				
				
				// tercer paso de la actualizacin.
				insNuevosProds= new TareaAsincInsertarProductosNuevos();
				insNuevosProds.execute();	
				
				
				//Toast.makeText(ActualizarBdActivity.this, "Tarea finalizada!",Toast.LENGTH_SHORT).show();
			}
		}

		@Override
		protected void onCancelled() {
			pDialog.dismiss();
			pDialog=null;

			//Toast.makeText(ActualizarBdActivity.this, "Tarea cancelada!",Toast.LENGTH_SHORT).show();
		}

	
	}
	
	
	
	/**
	 * Tarea asncrona para insertar en la base de datos del dispositivo los nuevos
	 * productos recibidos previamente.
	 */
	private class TareaAsincInsertarProductosNuevos extends
			AsyncTask<Void, Integer, Boolean> {


		@Override
		protected Boolean doInBackground(Void... params) {

	    	boolean resul = true;
	    	Producto prodAct=null;
			try
	        {

				if ((productos!=null) && (productos.length>0)) {
					for(int i=0; i<productos.length; i++)
		        	{
		        		prodAct=productos[i];
		        		if (db.insertarProductoGlobal(prodAct.getId(), prodAct.getCodigo(), prodAct.getDescripcion(), prodAct.getIngredientes(), prodAct.getFabricante())>0){
		        			publishProgress(1);
		        		}
		        	}
				}
	        }
	        catch(Exception ex)
	        {
	        	Log.e("InsProds","Error!", ex);
	        	resul = false;
	        } 	 
	        return resul;
		}

		@Override
		protected void onProgressUpdate(Integer... values) {
			int progreso = values[0].intValue();
			pDialog.setProgress(progreso);
		}

		@Override
		protected void onPreExecute() {

			pDialog = new ProgressDialog(ActualizarBdActivity.this);
			pDialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
			pDialog.setMessage(ActualizarBdActivity.this.getText(R.string.acvty_actualizarDb_RecibirWhile));
			pDialog.setCancelable(true);
			pDialog.setMax((numProdsGlobalNube - numProdsGlobal));
			
			
			pDialog.setOnCancelListener(new OnCancelListener() {
				@Override
				public void onCancel(DialogInterface dialog) {
					TareaAsincInsertarProductosNuevos.this.cancel(true);
				}
			});

			pDialog.setProgress(0);
			pDialog.show();
		}

		@Override
		protected void onPostExecute(Boolean result) {
			pDialog.dismiss();
			pDialog=null;

			if (result) {
				// mostrar resumen del estado
				stActualizacion = new TareaAsincEstadoActulizacion();
				stActualizacion.execute();
				
				//Toast.makeText(ActualizarBdActivity.this, "Tarea finalizada!",Toast.LENGTH_SHORT).show();
			}
		}

		@Override
		protected void onCancelled() {
			pDialog.dismiss();
			pDialog=null;

			//Toast.makeText(ActualizarBdActivity.this, "Tarea cancelada!",Toast.LENGTH_SHORT).show();
		}

	
	}	
	
	/**
	 * Tarea asncrona para calcular el estado de las actualizaciones
	 * dispositivo.
	 */
	private class TareaAsincEstadoActulizacion extends
			AsyncTask<Void, Integer, Boolean> {



		@Override
		protected Boolean doInBackground(Void... params) {

			try
			{
				//Actualizar contadores y parametros. - Conectar al servidor en la nube.
				numProdsGlobalNube=numProdsGlobalNube();

			} catch (Exception ex) {
				Log.e("TareaAsincEstadoActulizacion", "Error!", ex);
			} 
			
			if (isCancelled())
				return false;
			
			return true;
		}

		@Override
		protected void onProgressUpdate(Integer... values) {
		}

		@Override
		protected void onPreExecute() {

			pDialog = new ProgressDialog(ActualizarBdActivity.this);
			pDialog.setTitle(ActualizarBdActivity.this.getText(R.string.acvty_actualizarDb_CalcStatus));
			pDialog.setMessage(ActualizarBdActivity.this.getText(R.string.acvty_actualizarDb_CalcStatusWhile));
			pDialog.setIndeterminate(true);
			pDialog.setCancelable(true);			
			
			pDialog.setOnCancelListener(new OnCancelListener() {
				@Override
				public void onCancel(DialogInterface dialog) {
					TareaAsincEstadoActulizacion.this.cancel(true);
				}
			});

	
			pDialog.show();
			
			
			try
			{
				//Actualizar contadores y parametros.
				numProdsGlobal=(int) db.numProductosGlobal();
				numProdsLocal=(int) db.numProductosLocal();
				ultimaActualizacion=db.getUltimaActualizacion();

			} catch (Exception ex) {
				Log.e("TareaAsincEstadoActulizacion", "Error!", ex);
			}			
		}

		@Override
		protected void onPostExecute(Boolean result) {
			
			pDialog.dismiss();
			pDialog=null;
			if (result) {
				
				
				//Mostrar los datos de actualizacin aqu.
				txtUltAct.setText(ultimaActualizacion);
				txtTotalProdsNube.setText("" + numProdsGlobalNube);
				txtTotalProdsDisp.setText("" + numProdsGlobal);
				txtProdsPendRec.setText("" + (numProdsGlobalNube - numProdsGlobal));
				txtPendCompartir.setText("" + numProdsLocal );
			}
		}

		@Override
		protected void onCancelled() {
			pDialog.dismiss();
			pDialog=null;

		}




	}	
	/**
	 * @return devuelve el nmero de productos que existen en el servidor global - Nube.
	 */
	private int numProdsGlobalNube(){
		int resul=0;

		HttpClient httpClient = new DefaultHttpClient();


		try {
			HttpGet del = new HttpGet("http://ingredientapp.azurewebsites.net/api/ProductosApi/"+URLEncoder.encode("{count}","UTF-8"));
			del.setHeader("content-type", "application/json");

        	HttpResponse resp = httpClient.execute(del);
        	resul = Integer.parseInt(EntityUtils.toString(resp.getEntity())) ;

		} catch (Exception ex) {
			Log.e("ServicioRest", "Error!", ex);
			resul = 0;
		}			
		
		return resul;
	}

}
