package net.ase.ingredientapp;

import java.text.SimpleDateFormat;
import java.util.Date;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.database.sqlite.SQLiteStatement;
import android.util.Log;

//maneja los datos de las bases de datos de la aplicacion
public class AdaptadorBD {
	
	private boolean IsOpen=false;
	
	private static final String TAG = "AdaptadorBD";
	private static final String NOMBRE_BASEDATOS = "IngAppBD";
	private static final int VERSION_BASEDATOS = 1;

	//TABLA DE FILTROS
	public static final String FLTR_COL_IDFILA = "_id";
	public static final String FLTR_COL_TEXTO = "texto";
	private static final String TABLA_FILTROS = "filtros";
	private static final String CREAR_TABLA_FILTROS = "create table " + TABLA_FILTROS + " (" + FLTR_COL_IDFILA +  " integer primary key autoincrement," + FLTR_COL_TEXTO + " text not null);";
	
	//TABLA PRODUCTOS LOCAL
	public static final String PROD_COL_IDFILA_LOCAL = "_id";
	public static final String PROD_COL_CODIGO = "codigo";
	public static final String PROD_COL_DESC = "descripcion";
	public static final String PROD_COL_INGRE = "ingredientes";
	public static final String PROD_COL_FABRI = "fabricante";
	private static final String TABLA_PRODUCTOS_LOCAL = "productosLocal";
	private static final String CREAR_TABLA_PRODUCTOS_LOCAL = "create table " + TABLA_PRODUCTOS_LOCAL + " ("+ 
								PROD_COL_IDFILA_LOCAL + " integer primary key autoincrement, " + 
								PROD_COL_CODIGO + " text not null unique, " +
								PROD_COL_DESC + " text not null, " +
								PROD_COL_INGRE + " text not null, " +
								PROD_COL_FABRI +" text);";
	
	//TABLA PRODUCTOS GLOBAL
	public static final String PROD_COL_IDFILA_GLOBAL = "_id";
	private static final String TABLA_PRODUCTOS_GLOBAL = "productosGlobal";
	private static final String CREAR_TABLA_PRODUCTOS_GLOBAL = "create table " + TABLA_PRODUCTOS_GLOBAL + " ("+ 
								PROD_COL_IDFILA_LOCAL + "  integer primary key not null, " + 
								PROD_COL_CODIGO + " text not null unique, " +
								PROD_COL_DESC + " text not null, " +
								PROD_COL_INGRE + " text not null, " +
								PROD_COL_FABRI +" text);"; 

	//TABLA PARAMETROS
	public static final String PARAM_COL_IDFILA= "_id";
	public static final String PARAM_COL_ULTACT = "ultimaactualizacion";
	public static final String PARAM_COL_PARAM1 = "param1";
	public static final String PARAM_COL_PARAM2 = "param2";
	private static final String TABLA_PARAMETROS = "parametros";
	private static final String CREAR_TABLA_PARAMETROS = "create table " + TABLA_PARAMETROS + " ("+ 
								PARAM_COL_IDFILA + "  integer primary key not null, " +					
								PARAM_COL_PARAM1 + " text, " +
								PARAM_COL_PARAM2 + " text, " +
								PARAM_COL_ULTACT +" text);"; 
	
	
	private final Context context;
	private DatabaseHelper DBHelper;
	private SQLiteDatabase db;

	public AdaptadorBD(Context ctx) {
		this.context = ctx;
		DBHelper = new DatabaseHelper(context);
	}

	/**
	 * Esta clase agrupa todos los mtodos que acceden a la base de datos de la aplicacin.
	 * @author Alex
	 *
	 */
	private static class DatabaseHelper extends SQLiteOpenHelper {
		DatabaseHelper(Context context) {
			super(context, NOMBRE_BASEDATOS, null, VERSION_BASEDATOS);
		}

		@Override
		public void onCreate(SQLiteDatabase db) {
			try {
				db.execSQL(CREAR_TABLA_FILTROS);
				db.execSQL(CREAR_TABLA_PRODUCTOS_LOCAL);
				db.execSQL(CREAR_TABLA_PRODUCTOS_GLOBAL);
				db.execSQL(CREAR_TABLA_PARAMETROS);
				
				//parmetros de la aplicacin
				SimpleDateFormat df1 = new SimpleDateFormat("dd/MM/yyyy");
				String auxDate= df1.format(new Date());

				ContentValues valoresIniciales = new ContentValues();
				valoresIniciales.put(PARAM_COL_IDFILA, "1");
				valoresIniciales.put(PARAM_COL_ULTACT, auxDate);
				db.insert(TABLA_PARAMETROS, null, valoresIniciales);				
				
			} catch (SQLException e) {
				Log.e("onCreate", "Error!", e);
			}
		}

		@Override
		public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
			
			try {
				Log.w(TAG, "Actualizando base de datos de versin " + oldVersion + " a "
						+ newVersion + ", lo que destruir todos los viejos datos");
				db.execSQL("DROP TABLE IF EXISTS " + TABLA_FILTROS);
				db.execSQL("DROP TABLE IF EXISTS " + TABLA_PRODUCTOS_LOCAL);
				db.execSQL("DROP TABLE IF EXISTS " + TABLA_PRODUCTOS_GLOBAL);
				db.execSQL("DROP TABLE IF EXISTS " + TABLA_PARAMETROS);
				
				onCreate(db);			
				
			} catch (SQLException e) {
				Log.e("onUpgrade", "Error!", e);
			}			
			

		}
	}

	/**
	 * abrir la base de datos---
	 * @return
	 * @throws SQLException
	 */
	public AdaptadorBD abrir() throws SQLException {
		db = DBHelper.getWritableDatabase();
		this.IsOpen=true;
		return this;
	}

	/**
	 * cerrar la base de datos 
	 */
	public void cerrar() {
		DBHelper.close();
		db=null;
		this.IsOpen=false;
	}

	//getters setters
	public boolean IsOpen() {
		return this.IsOpen;
	}	
	
	/**
	 * insertar un filtro en la base de datos---
	 * @param texto el filtro a insertar.
	 * @return el identificador dentro de la tabla de filtros
	 */
	public long insertarFiltro(String texto) {
		long res=-1;
		try
		{
			ContentValues valoresIniciales = new ContentValues();
			valoresIniciales.put(FLTR_COL_TEXTO, texto);
			res= db.insert(TABLA_FILTROS, null, valoresIniciales);			
		} catch (Exception ex){
			Log.e("insertarFiltro", "Error!", ex);
			res=-1;
		}
		return res;

	}

	/**
	 * Elimina todos los filtros actuales
	 * 
	 */
	public void resetFiltros() {
		 db.execSQL("delete from " + TABLA_FILTROS);
	}

	

	/**
	 * Borra un filtro en concreto 
	 * @param idFila el identificador del fitro en la tabla
	 * @return cierto si se borra correctamente. Falso en caso contrario
	 */
	public boolean borrarFiltro(long idFila) {
		boolean res=false;
		try
		{
			res= db.delete(TABLA_FILTROS, FLTR_COL_IDFILA + "=" + idFila, null) > 0;	
		} catch (Exception ex){
			Log.e("borrarFiltro", "Error!", ex);
			res=false;
		}
		return res;
		
	}

	/**
	 * 
	 * @return Devuelve un cursor con todos los filtros definidos por el usuario ordenados alfabticamente.
	 */
	public Cursor obtenerTodosLosFiltros() {
		Cursor mCursor=null;
		try{
			mCursor =db.query(TABLA_FILTROS, new String[] { FLTR_COL_IDFILA, FLTR_COL_TEXTO }, null, null, null, null, FLTR_COL_TEXTO + " ASC");
			
		}catch (Exception ex){
			Log.e("obtenerTodosLosFiltros", "Error!", ex);
			mCursor=null;
		}
		return mCursor;
		
	}

	/**
	 * Recupera un filtro en concreto
	 * @param idFila el identificador del fitro en la tabla
	 * @return  Devuelve un cursor con todos los datos
	 * @throws SQLException
	 */
	public Cursor obtenerFiltro(long idFila) throws SQLException {

		Cursor mCursor=null;
		try{
				mCursor = db.query(true, TABLA_FILTROS, new String[] {
					FLTR_COL_IDFILA, FLTR_COL_TEXTO }, FLTR_COL_IDFILA + "=" + idFila,
					null, null, null, null, null);
			if (mCursor != null) {
				mCursor.moveToFirst();
			}
			
		}catch (Exception ex){
			Log.e("obtenerFiltro", "Error!", ex);
			mCursor=null;
		}
		return mCursor;
	}

	/**
	 * Actualizar un filtro
	 * @param idFila el identificador del fitro en la tabla
	 * @param texto el nuevo valor del filtro
	 * @return cierto si se actualiza correctamente. Falso en caso contrario
	 */
	public boolean actualizarFiltro(long idFila, String texto) {
		
		boolean res=false;
		try
		{
			ContentValues args = new ContentValues();
			args.put(FLTR_COL_IDFILA, idFila);
			args.put(FLTR_COL_TEXTO, texto);
			res= db.update(TABLA_FILTROS, args, FLTR_COL_IDFILA + "=" + idFila, null) > 0;
			
		} catch (Exception ex){
			Log.e("actualizarFiltro", "Error!", ex);
			res=false;
		}
		
		return res;
		
		

	}
	
	//	---------------------------	
	//  ------- PRODUCTOS ---------
	//	---------------------------

	
	/**
	 * Borra todos los productos de las bases de datos del dispositivo
	 */
	public void resetProducto() {
		 db.execSQL("delete from " + TABLA_PRODUCTOS_LOCAL);
		 db.execSQL("delete from " + TABLA_PRODUCTOS_GLOBAL);
	}

	
	
	/**
	 * Busca un producto en la BD local
	 * @param codigo el codigo del producto 
	 * @return Devuelve un cursor con todos los datos
	 */
	public Cursor buscarProductoLocal(String codigo) {

		Cursor mCursor=null;
		try
		{
			mCursor = db.query(true, TABLA_PRODUCTOS_LOCAL, new String[] {
					PROD_COL_CODIGO, PROD_COL_DESC,PROD_COL_INGRE, PROD_COL_FABRI}, PROD_COL_CODIGO + "=" + codigo,
					null, null, null, null, null);
			if (mCursor != null) {
				mCursor.moveToFirst();
			}
			
		}catch (Exception ex){
			Log.e("buscarProductoLocal", "Error!", ex);
			mCursor=null;
		}
		return mCursor;
	}


	/**
	 * buscar un producto en la BD global
	 * @param codigo cdigo el codigo del producto 
	 * @return Devuelve un cursor con todos los datos
	 */
	public Cursor buscarProductoGlobal(String codigo) {

		Cursor mCursor=null;
		try
		{
			mCursor = db.query(true, TABLA_PRODUCTOS_GLOBAL, new String[] {
					PROD_COL_CODIGO, PROD_COL_DESC,PROD_COL_INGRE, PROD_COL_FABRI}, PROD_COL_CODIGO + "=" + codigo,
					null, null, null, null, null);
			if (mCursor != null) {
				mCursor.moveToFirst();
			}
			return mCursor;

			
		}catch (Exception ex){
			Log.e("buscarProductoGlobal", "Error!", ex);
			mCursor=null;
		}
		return mCursor;		
		
		
	}
	
	/**
	 * Elimina un registo de la tabla de productos local
	 * @param idFila el identificador del registro a eliminar.
	 * @return devuelve un booleano indicando si se borr o no el registr.
	 */
	public boolean borrarProductoLocal(long idFila) {
		
		boolean res=false;
		try
		{
			res=db.delete(TABLA_PRODUCTOS_LOCAL, PROD_COL_IDFILA_LOCAL + "=" + idFila, null) > 0;
		}catch (Exception ex){
			Log.e("borrarProductoLocal", "Error!", ex);
			res=false;
		}
		
		return res;
	}
	
	/**
	 * insertar un producto en la base de datos local 
	 * @param codigo el codigo del producto
	 * @param descripcion descripcion del producto
	 * @param ingredientes ingredientes del producto
	 * @param fabricante fabricante del producto
	 * @return
	 */
	public long insertarProductoLocal(String codigo, String descripcion, String ingredientes, String fabricante) {
		ContentValues valoresIniciales = new ContentValues();
		valoresIniciales.put(PROD_COL_CODIGO ,codigo);
		valoresIniciales.put(PROD_COL_DESC ,descripcion);
		valoresIniciales.put(PROD_COL_INGRE ,ingredientes);
		valoresIniciales.put(PROD_COL_FABRI ,fabricante);		
		
		return db.insert(TABLA_PRODUCTOS_LOCAL, null, valoresIniciales);
	}

	/**
	 * insertar un producto en la base de datos global 
	 * @param codigo el codigo del producto
	 * @param descripcion descripcion del producto
	 * @param ingredientes ingredientes del producto
	 * @param fabricante fabricante del producto
	 * @return
	 */
	public long insertarProductoGlobal(int id, String codigo, String descripcion, String ingredientes, String fabricante) {
		ContentValues valoresIniciales = new ContentValues();
		valoresIniciales.put(PROD_COL_IDFILA_GLOBAL,id);
		valoresIniciales.put(PROD_COL_CODIGO ,codigo);
		valoresIniciales.put(PROD_COL_DESC ,descripcion);
		valoresIniciales.put(PROD_COL_INGRE ,ingredientes);
		valoresIniciales.put(PROD_COL_FABRI ,fabricante);		
		
		return db.insert(TABLA_PRODUCTOS_GLOBAL, null, valoresIniciales);
	}
	
	/**
	 * 
	 * @return devuelve un cursor con tdos los productos de la tabla local de productos.
	 */
	public Cursor obtenerTodosLosProductosLocal() {
		
		Cursor mCursor=null;
		try
		{
			mCursor = db.query(true, TABLA_PRODUCTOS_LOCAL, new String[] {
					PROD_COL_IDFILA_LOCAL, PROD_COL_CODIGO, PROD_COL_DESC,PROD_COL_INGRE, PROD_COL_FABRI},
					null,null, null, null, null, null);
			if (mCursor != null) {
				mCursor.moveToFirst();
			}
			return mCursor;
			
		}catch (Exception ex) {
			Log.e("obtenerTodosLosProductosLocal", "Error!", ex);
			mCursor=null;
			
		}

		return mCursor;	
		}
	
	/**
	 * 
	 * @return Devuelve el nmero de elementos de la tabla local (pendientes de compartir) de productos
	 */
	public long numProductosLocal () throws SQLException {
		
		 SQLiteStatement s = db.compileStatement("select count(*) from " + TABLA_PRODUCTOS_LOCAL);
		 return s.simpleQueryForLong();		
	}
	/**
	 * 
	 * @return Devuelve el nmero de elementos de la tabla local de productos. La que se sincroniza con la global de la nube.
	 */
	public long numProductosGlobal () throws SQLException {
		 SQLiteStatement s = db.compileStatement("select count(*) from " + TABLA_PRODUCTOS_GLOBAL);
		 return s.simpleQueryForLong();		
	}	
	
	/**
	 * Devuelve la fecha de la ltima ejecucin de la actualizacin
	 */
	public String getUltimaActualizacion(){
		String res="";
		
		Cursor mCursor = db.query(true, TABLA_PARAMETROS, new String[] {
				PARAM_COL_ULTACT}, PARAM_COL_IDFILA + "= 1" ,
				null, null, null, null, null);
		if (mCursor != null) {
			if (mCursor.moveToFirst()){
				res=mCursor.getString(0);
			}
		}
		
		return res;
	}

	/**
	 * Establece la fecha de la ltima ejecucin de la actualizacin
	 */
	public boolean setUltimaActualizacion(String date){
		
		
		ContentValues args = new ContentValues();

		args.put(PARAM_COL_ULTACT, date);
		return db.update(TABLA_PARAMETROS, args, PARAM_COL_IDFILA + "=1" , null) > 0;

	}

	
	/**
	 * Devuelve el identificador de orden del ltimo producto recibido del servidor central
	 */
	public long getUltimoProductoRecibido(){
		
		 SQLiteStatement s = db.compileStatement("select max (" + PROD_COL_IDFILA_LOCAL + ") from " + TABLA_PRODUCTOS_GLOBAL);
		 return s.simpleQueryForLong();	

	}	
	
}
