package net.ase.ingredientapp;



import java.util.ArrayList;
import java.util.List;

import android.app.Activity;
import android.app.ActionBar;
import android.app.AlertDialog;
import android.app.Fragment;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.database.Cursor;
import android.os.Bundle;
import android.speech.RecognizerIntent;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.view.inputmethod.EditorInfo;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.TextView.OnEditorActionListener;
import android.os.Build;

/**
 * Esta clase encapsula todos lo procesos de busqueda de productos en la base de datos del dispositivo.
 * @author Alex
 *
 */
public class BuscarProductoActivity extends Activity {

	
	//Cdigo del intent de reconocimiento de voz
	private static final int VOICE_RECOGNITION_REQUEST_CODE = 1234;
	
	// Objetos de la vista
	EditText txtBuscaProd;
	ImageButton btnEscanerCB;
	Producto prod;
	ImageButton speakButton;
	// creamos una instancia de la BD
	private AdaptadorBD db;
	

	
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_buscar_producto);

		
		//creamos una instancia de la BD
		db = new AdaptadorBD(this);
		prod=new Producto(this);
		

		
		//Preparando el intent de reconocimiento de voz
        // Get display items for later interaction
		speakButton = (ImageButton) findViewById(R.id.btn_speak_desc);
        // Check to see if a recognition activity is present
		PackageManager pm = getPackageManager();
        List<ResolveInfo> activities = pm.queryIntentActivities(
                new Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH), 0);
        if (activities.size() != 0) {
            speakButton.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {
                	startVoiceRecognitionActivity();
                }


           });
        } else {
            speakButton.setEnabled(false);
        }
		
        // Referenciar los objetos de la vista
		txtBuscaProd = (EditText) findViewById(R.id.txtIngreProdDb);
		txtBuscaProd.setOnEditorActionListener(new OnEditorActionListener() {
				
			@Override
		    public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
		        boolean handled = false;
		        if (actionId == EditorInfo.IME_ACTION_SEARCH) {
		        	
		        	String text=v.getText().toString();

					if (text != null && text.length() > 0){
						procesoBusquedaProducto(text);
						
					}		        	
		        	
		        	
		        	
		            handled = true;
		        }
		        return handled;
		    }
		});			
		
		btnEscanerCB = (ImageButton) this.findViewById(R.id.btnEscanerCB);
		btnEscanerCB.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View view) {

				Intent intent = new Intent("net.ase.ingredientapp.SCAN");
				intent.putExtra("SCAN_MODE", "ONE_D_FORMATS");
				startActivityForResult(intent, 0);			
				
	        	
			}
		});			
		
		
	
		
		
	
	}

    /**
     * Fire an intent to start the speech recognition activity.
     */
    private void startVoiceRecognitionActivity() {
        Intent intent = new Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH);

        // Specify the calling package to identify your application
        intent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, getClass().getPackage().getName());

        // Display an hint to the user about what he should say.
        intent.putExtra(RecognizerIntent.EXTRA_PROMPT, this.getString(R.string.txtBuscaProdHint));

        // Given an hint to the recognizer about what the user is going to say
        intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL,
                RecognizerIntent.LANGUAGE_MODEL_FREE_FORM);

        // Specify how many results you want to receive. The results will be sorted
        // where the first result is the one with higher confidence.
        intent.putExtra(RecognizerIntent.EXTRA_MAX_RESULTS, 1);

        startActivityForResult(intent, VOICE_RECOGNITION_REQUEST_CODE);
    }
	
    /**
     * 
     * @param val una cadena a evaluar como numrica.
     * @return devuelve cierto si se evalua como numrica. Falso si no es posible
     */
    private boolean isNumber(String val){
    	
    	try
    	{
    		Long.parseLong(val);
    		
    	} catch (Exception ex){
    		return false;
    	}
    	
    	return true;
    }
    
	/**
	 * Trata todo el caso de busqueda de producto, muestra de resultados y dems.
	 * 
	 */
	public void procesoBusquedaProducto(final String codigo){
		
		Cursor producto=null;
		
		
		if (codigo != null && !codigo.isEmpty()) {
			
			if (isNumber(codigo)){
				producto=prod.productoId(codigo);
				if ( (producto!= null) && (producto.moveToFirst()) ){
					
					mostrarProductoEncontrado(codigo);
					
				}
				else {
	
	
					// Preguntar si creamos nuevo producto.
					AlertDialog.Builder alert = new AlertDialog.Builder(this);
					alert.setTitle(this.getString(R.string.acvty_buscarProd_ProdNoEncontrado));
					alert.setMessage(this.getString(R.string.acvty_buscarProd_PreguntaCrearNuevo));
					alert.setPositiveButton(this.getString(R.string.acvty_buscarProd_Ok), new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
						mostrarProductoNuevo(codigo);
					  }
					});
	
					alert.setNegativeButton(this.getString(R.string.acvty_buscarProd_Cancel), new DialogInterface.OnClickListener() {
					  public void onClick(DialogInterface dialog, int whichButton) {
					    // Cancelado.
					  }
					});
	
					alert.show();				
					
				}
				
			}
			else{
				Toast.makeText(this,this.getString(R.string.acvty_buscarProd_NoNum)  , Toast.LENGTH_LONG).show();
				
			}
			
			
			
		}
	}
	/**
	 * Pregunta al usuario si quiere crear un producto nuevo y en caso 
	 * afirmativo muestra la nueva activity del producto nuevo.
	 * 
	 */
	private void mostrarProductoNuevo(String codigo) {

		  // Creando el nuevo producto
    	Intent intent = new Intent(this, ProductoNuevoActivity.class);
    	
    	//Creamos la informacin a pasar entre actividades
    	Bundle b = new Bundle(); 
    	b.putString(AdaptadorBD.PROD_COL_CODIGO,codigo);

    	
    	//Aadimos la informacin al intent
    	intent.putExtras(b);
    	
    	//Iniciamos la nueva actividad
        startActivity(intent);				
		
		
	}

	/**
	 * Crea una nueva activity donde muestra los datos del producto encontrado.
	 * 
	 */
	public void mostrarProductoEncontrado(String codigo){

		
		
		  // Creando el nuevo producto
    	Intent intent = new Intent(this, ProductoEncontradoActivity.class);
    	
    	//Creamos la informacin a pasar entre actividades
    	Bundle b = new Bundle(); 
    	b.putString(AdaptadorBD.PROD_COL_CODIGO,codigo);

    	
    	//Aadimos la informacin al intent
    	intent.putExtras(b);
    	
    	//Iniciamos la nueva actividad
        startActivity(intent);			
		

	}
	

	

	/**
	 * Recibe los datos de la activitys que se lanzaron en para obtener un resultado.
	 * Cada una de ellas tiene un cdigo que las identifica. 
	 * En este caso hay dos
	 * -el reconocimiento de voz
	 * -el reconocimientod de cdigos de barras.
	 */
	public void onActivityResult(int requestCode, int resultCode, Intent intent) {
		    if (requestCode == 0) {
		        if (resultCode == RESULT_OK) {
		            String contenido = intent.getStringExtra("SCAN_RESULT");
		            //String formato = intent.getStringExtra("SCAN_RESULT_FORMAT");
					
					if (contenido != null && contenido.length() > 0){
						procesoBusquedaProducto(contenido);
						
					}			            
		            
		        } else if (resultCode == RESULT_CANCELED) {
		            // Si se cancelo la captura.
		        }
		    } else if (requestCode == VOICE_RECOGNITION_REQUEST_CODE && resultCode == RESULT_OK) {
	            // Fill the list view with the strings the recognizer thought it could have heard
	            ArrayList<String> matches = intent.getStringArrayListExtra(
	                    RecognizerIntent.EXTRA_RESULTS);
	            	            
	            txtBuscaProd.setText(matches.toString().replace(" ", "").replace("[", "").replace("]", ""));
	            procesoBusquedaProducto(txtBuscaProd.getText().toString());

	        }
		    
		}



}
