package net.ase.ingredientapp;

import java.util.ArrayList;
import java.util.Locale;

import android.R.string;
import android.app.Activity;
import android.app.ActionBar;
import android.app.Fragment;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.widget.ImageButton;
import android.widget.ListView;
import android.widget.EditText;
import android.widget.Toast;
import android.os.Build;

/**
 * Esta clase es la que interactua con las preferencias, aade, edita y elimina.
 * @author Alex
 *
 */
public class PreferenciasActivity extends Activity implements OnClickListener {
	

	
	private ListView lista;
	private ArrayList<PreferFiltro> arrayFiltros;
	private AdapterPreferFiltro adapter;

	private EditText txtAddPref;
	private ImageButton btnAddPref;
	
	// creamos una instancia de la BD
	private AdaptadorBD db;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_preferencias);

		
		
		// referenciamos los views de la activity
		lista = (ListView) findViewById(R.id.listaFiltros);
        txtAddPref = (EditText) this.findViewById(R.id.txtAddPref);
        btnAddPref = (ImageButton) this.findViewById(R.id.btnAddPref);
		btnAddPref.setOnClickListener(this);        
		 
		//creamos una instancia de la BD
		arrayFiltros = new ArrayList<PreferFiltro>();
		db = new AdaptadorBD(this);
		
		db.abrir();
		//db.resetFiltros();
		arrayFiltros = PreferFiltro.cursorToArrayFiltros(db
				.obtenerTodosLosFiltros(),arrayFiltros);
		db.cerrar();
		
        


        
        // Creo el adapter personalizado
        adapter = new AdapterPreferFiltro(this, arrayFiltros);
 
        // Lo aplico
        lista.setAdapter(adapter);
		

	
	}


	@Override
	public void onClick(View v) {
		// Creamos un switch para comprobar que view de la activity lanz el
		// evento
		switch (v.getId()) {
		case R.id.btnAddPref:

			String Aux;
			Aux = txtAddPref.getText().toString();
			aadirPreferencia(Aux);

			break;

		}
		
		
	}

	/**
	 * Actualiza la lista de filtros desde la BD.
	 */
	public void actulizarLista(){
		db.abrir();
		arrayFiltros = PreferFiltro.cursorToArrayFiltros(db
				.obtenerTodosLosFiltros(),arrayFiltros);
		db.cerrar();	
		adapter.notifyDataSetChanged();
	}
	public boolean aadirPreferencia(String val){
		boolean res;
		res=false;
		
		long id;
		
		
		val = txtAddPref.getText().toString().toUpperCase();
		if (val != null && !val.isEmpty()) {

			PreferFiltro addFiltro;
			addFiltro = new PreferFiltro(val, arrayFiltros.size());
			if (arrayFiltros.contains(addFiltro) == false) {

				db.abrir();
				id = db.insertarFiltro(val);
				db.cerrar();

				if (id > 0) {
					res=true;
					actulizarLista();
					Toast.makeText(this, "Preferencia aadida ",
							Toast.LENGTH_SHORT).show();
					txtAddPref.setText("");
				}
			}
		}		
	
		return res;
	}
	/**
	 * Elimina una preferencia.
	 * @param pos el id de la preferencia
	 * @return cierto si se ejecuta correctamente falso en cualquier otra caso
	 */
	public boolean eliminarPreferencia(long pos){
		boolean res;
		res=false;
		db.abrir();
		if (db.borrarFiltro(pos)){
			Toast.makeText(this, "Preferencia eliminada",
					Toast.LENGTH_SHORT).show();			
			actulizarLista();
		}
		db.cerrar();
		return res;
	}
	
	/**
	 * Actualiza una preferencia
	 * @param pos el id de la preferencia
	 * @param val el nuevo valor
	 * @return cierto si se ejecuta correctamente falso en cualquier otra caso
	 */
	public boolean actualizarPreferencia(long pos, String val){
		boolean res;
		res=false;

		if (val != null && !val.isEmpty()) {

			PreferFiltro addFiltro;
			addFiltro = new PreferFiltro(val, arrayFiltros.size());
			if (arrayFiltros.contains(addFiltro) == false) {

				db.abrir();
				if (db.actualizarFiltro(pos, val)){
					res=true;
					actulizarLista();
					Toast.makeText(this, "Preferencia actualizada",
							Toast.LENGTH_SHORT).show();
					txtAddPref.setText("");
				}
				db.cerrar();

			}
		}		

		return res;
	}	
}
