package net.ase.ingredientapp;


import android.content.Context;
import android.database.Cursor;

/**
 * Representa un producto dentro de la aplicacin
 * @author Alex
 *
 */
public class Producto {

	
	private int id;
	private String codigo;
	private String descripcion;
	private String ingredientes;
	private String fabricante;	
	
	
	// creamos una instancia de la BD
	private AdaptadorBD db;
	private final Context context;
	
	public Producto (Context ctx) {
		//creamos una instancia de la BD
		this.context = ctx;
		db = new AdaptadorBD(this.context);			
	}
	
	
	public Producto() {
		context=null; 
	}


	//getters setters
	public int getId() {
		return this.id;
	}
	public void setId(int value) {
		this.id = value;
	}	

	public String getCodigo() {
		return this.codigo;
	}
	public void setCodigo(String value) {
		this.codigo = value;
	}

	public void setDescripcion(String value) {
		this.descripcion = value;
	}	
	public String getDescripcion() {
		return this.descripcion;
	}
	
	public void setIngredientes(String value) {
		this.ingredientes = value;
	}	
	public String getIngredientes() {
		return this.ingredientes;
	}
	
	public void setFabricante(String value) {
		this.fabricante = value;
	}	
	public String getFabricante() {
		return this.fabricante;
	}

	
	
	
	
	/**
	 * Busca un producto en las BD del dispositivo a partir de su cdigo de barras.
	 * 
	 * @param codigo el cdigo de barras
	 * @return devuelve un Cursor con los datos del producto o null
	 */
	public Cursor productoId(String codigo){
		Cursor c=null;
		
		db.abrir();

		c=db.buscarProductoLocal(codigo);
		if (! c.moveToFirst()) {
			c=db.buscarProductoGlobal(codigo);
		}
		
		db.cerrar();
		
		return c;
	}
	/**
	 * Devuelve un booleano cierto si el producto cumple con las preferencias.
	 * falso si no las cumple.
	 * 
	 * Si no existen preferencias devuleve siempre cierto.
	 * 
	 * @param ingredientes los ingredientes del producto en la BD del disposivo.
	 * 
	 * @return cierto o falso dependiendo de las preferencias del usuario y los ingredientes.
	 */	
	public boolean cumplePreferencias(String ingredientes){
		boolean res=true;
		Cursor curPrefs=null;
		String filtro="";
		db.abrir();
		
		curPrefs=db.obtenerTodosLosFiltros();
		if (curPrefs.moveToFirst()) {
		     //Recorremos el cursor hasta que no haya ms registros
		     do {
		    	 filtro=curPrefs.getString(1);
		    	 if (ingredientes.contains(filtro)){
		    		 res=false;
		    	 }
		     } while(curPrefs.moveToNext() && (res));
		}
		
		db.cerrar();
		
		return res;
	}
	
	
	
	
}
