package net.ase.ingredientapp;

import java.util.ArrayList;
import java.util.List;

import android.app.Activity;
import android.app.ActionBar;
import android.app.Fragment;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.os.Bundle;
import android.speech.RecognizerIntent;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.TextView;
import android.os.Build;

/**
 * Esta clase se encarga de crear nuevos productos cuando no existen en la base de datos del dispositivo.
 * @author Alex
 *
 */
public class ProductoNuevoActivity extends Activity  implements OnClickListener {

	//Cdigo del intent de reconocimiento de voz
	private static final int VOICE_RECOGNITION_REQUEST_CODE_DESCRIPCION = 1234;	
	private static final int VOICE_RECOGNITION_REQUEST_CODE_INGREDIENTES = 4321;
	
	//Objetos de la vista
	TextView txtCodigoProd;
	EditText txtDescProd;
	EditText txtIngreProd;
	Button btnCancel;
	Button btnListo;
	String codigo="";

	ImageButton speakButtonDesc;
	ImageButton speakButtonIngr;	
	
	// Base de datos
	private AdaptadorBD db;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_producto_nuevo);


		//Preparando el intent de reconocimiento de voz
        // Get display items for later interaction
		speakButtonDesc = (ImageButton) findViewById(R.id.btn_speak_desc);
		speakButtonIngr = (ImageButton) findViewById(R.id.btn_speak_ingred);
        // Check to see if a recognition activity is present
		PackageManager pm = getPackageManager();
        List<ResolveInfo> activities = pm.queryIntentActivities(
                new Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH), 0);
        if (activities.size() != 0) {
        	
        	speakButtonDesc.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {
                	startVoiceRecognitionActivity(VOICE_RECOGNITION_REQUEST_CODE_DESCRIPCION,ProductoNuevoActivity.this.getString(R.string.acvty_producto_DescripcionProd));
                }


           });
        	speakButtonIngr.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View v) {
                	startVoiceRecognitionActivity(VOICE_RECOGNITION_REQUEST_CODE_INGREDIENTES,ProductoNuevoActivity.this.getString(R.string.acvty_producto_IngredientesHint));
                }


           });        	
        	
        } else {
        	speakButtonDesc.setEnabled(false);
        	speakButtonIngr.setEnabled(false);
        }		
		
		//creamos una instancia de la BD
		db = new AdaptadorBD(this);		
		
		
		// Referenciar los objetos de la vista
		txtCodigoProd = (TextView) findViewById(R.id.txtCodigoProd);
		txtDescProd = (EditText) findViewById(R.id.txtDescProd);
		txtIngreProd = (EditText) findViewById(R.id.txtIngreProd);
		btnCancel = (Button) findViewById(R.id.btnCancel);
		btnListo = (Button) findViewById(R.id.btnListo);
		
		// Registrar los handlers de los eventos de los botones
		btnCancel.setOnClickListener(this);
		btnListo.setOnClickListener(this);
		
		
        //Recuperamos la informacin pasada en el intent
        Bundle bundle = this.getIntent().getExtras();
        codigo = bundle.getString(AdaptadorBD.PROD_COL_CODIGO);
        
        txtCodigoProd.setText(codigo);
        

	}

    /**
     * Fire an intent to start the speech recognition activity.
     */
    private void startVoiceRecognitionActivity(int requestCode, String ExtraPrompt) {
        Intent intent = new Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH);
        
        // Specify the calling package to identify your application
        intent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, getClass().getPackage().getName());

        // Display an hint to the user about what he should say.
        intent.putExtra(RecognizerIntent.EXTRA_PROMPT, ExtraPrompt);

        // Given an hint to the recognizer about what the user is going to say
        intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL,
                RecognizerIntent.LANGUAGE_MODEL_FREE_FORM);

        // Specify how many results you want to receive. The results will be sorted
        // where the first result is the one with higher confidence.
        intent.putExtra(RecognizerIntent.EXTRA_MAX_RESULTS, 1);

        startActivityForResult(intent, requestCode);
    }	
	
	@Override
	public void onClick(View v) {
		// Creamos un switch para comprobar que view de la activity lanz el evento
		switch (v.getId()) {
		case R.id.btnCancel:
		        //Accin cancelada
				finActivity();
			break;
	
			
		case R.id.btnListo:
	        //Guardar producto
			if (guardarProductoLocal()){
				mostrarProductoEncontrado(txtCodigoProd.getText().toString().toUpperCase().trim());
				finActivity();
			}
		break;			

		}
		
	}

	/**
	 * Destruye la activity y vuelve a la anterior.
	 * 
	 */
	private void finActivity() {
		finish();
	}
	/**
	 * Valida la informacion del formulario y guarda el producto en la BD Local.
	 * @return
	 */
	private boolean guardarProductoLocal(){
		boolean res=true;
		String CodigoProd,DescProd,IngreProd;
		
		CodigoProd=txtCodigoProd.getText().toString().toUpperCase().trim();
		DescProd=txtDescProd.getText().toString().toUpperCase().trim();
		IngreProd=txtIngreProd.getText().toString().toUpperCase().trim();
		

		if (!(DescProd.length()>0)){
			txtDescProd.setError(this.getText(R.string.acvty_producto_New_campo_ko));
			res=false;
		}
		if (!(IngreProd.length()>0)){
			txtIngreProd.setError(this.getText(R.string.acvty_producto_New_campo_ko));
			res=false;
		}		
		
		if (res){
			if ( (CodigoProd.length()>0) && (DescProd.length()>0) && (IngreProd.length()>0)) {
				db.abrir();
				res=(db.insertarProductoLocal(CodigoProd, DescProd, IngreProd, "")>0);
				db.cerrar();
			}			
		}
		
		return res;
	}
	
	
	/**
	 * Crea una nueva activity donde muestra los datos del producto encontrado.
	 * 
	 */
	public void mostrarProductoEncontrado(String codigo){

		
		
		  // Creando el nuevo producto
    	Intent intent = new Intent(this, ProductoEncontradoActivity.class);
    	
    	//Creamos la informacin a pasar entre actividades
    	Bundle b = new Bundle(); 
    	b.putString(AdaptadorBD.PROD_COL_CODIGO,codigo);

    	
    	//Aadimos la informacin al intent
    	intent.putExtras(b);
    	
    	//Iniciamos la nueva actividad
        startActivity(intent);			
		

	}
	
	/**
	 * Recibe los datos de la activitys que se lanzaron en para obtener un resultado.
	 * Cada una de ellas tiene un cdigo que las identifica. 
	 * En este caso solo hay una
	 * -el reconocimiento de voz
	 */
	public void onActivityResult(int requestCode, int resultCode, Intent intent) {
		    if ( resultCode == RESULT_OK) {

		    	// Fill the list view with the strings the recognizer thought it could have heard
	            ArrayList<String> matches = intent.getStringArrayListExtra(
	                    RecognizerIntent.EXTRA_RESULTS);
	            
	            
	            String resVoice="";
	            String[] resVoiceArr = new String[matches.size()];
	            resVoiceArr = matches.toArray(resVoiceArr);

	            StringBuilder builder = new StringBuilder();
	            for(String s : resVoiceArr) {
	                builder.append(s);
	            }
	            resVoice= builder.toString().toUpperCase();	            
	            
		    	
		    	if (requestCode == VOICE_RECOGNITION_REQUEST_CODE_DESCRIPCION ){
	            	txtDescProd.setText(resVoice);
		    	}
	            
		    	if (requestCode == VOICE_RECOGNITION_REQUEST_CODE_INGREDIENTES ){
	            	txtIngreProd.setText(resVoice);
		    	}
	            
	        }
		    
		}	

}
